﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

//  B-Splines 
//  www.mosismath.com

namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        public const int order = 4;          // number of reference points (max 10)
        public const int datapoints = 1000; // number of datapoints for the interpolation

        public TextBox[,] tbm;
        public TextBox[] tbs;
        public TextBox[] tbx;

        public class TMatrix
        {
            public double[,] a;
            public double[] y;
            public double[] x;

            public TMatrix(int size)
            {
                a = new double[size, size];
                y = new double[size];
                x = new double[size];
            }
        }

        TMatrix m;
        double[] yp = new double[datapoints];

        double[] yp2 = new double[datapoints];
        double[] yp3 = new double[datapoints];
        double[] yp4 = new double[datapoints];

        double[] xp = new double[datapoints];
        double[] y_k = new double[order];
        double[] x_k = new double[order + 10];
        double[] a = new double[order];
        double[] b = new double[order];
        double[] c = new double[order];
        double[] d = new double[order];
        double[] h = new double[order-1];


       double CalcBinom(int m, int n)
       {
            int i;
            double numerator = 1;
            double denominator = 1;
            if (m >= (2 * n))
            {
                for (i = m; i > (m - n); i--)
                {
                    numerator = numerator * (double)i;
                }
                for (i = n; i >= 2; i--)
                {
                    denominator = denominator * (double)i;
                }
                return numerator / denominator;
            }
            return 0;
        }

       void Approximate(int nbOfPoints)
       {
           int i, k;
           double t1;
           double t2;

           for (k = 0; k <= nbOfPoints; k++)
           {
               yp[k] = 0;
               xp[k] = 0;
               t1 = 1 - (double)(k) / (double)(nbOfPoints);
               t2 = (double)(k) / (double)(nbOfPoints);
               for (i = 0; i < order; i++)
               {
                   yp[k] = yp[k] + y_k[i] * Math.Pow(t1, order - i - 1) * Math.Pow(t2, i) * b[i];
                   xp[k] = xp[k] + x_k[i] * Math.Pow(t1, order - i - 1) * Math.Pow(t2, i) * b[i];

               }
           }
       }

        public MainWin()
        {
            int i;
            m = new TMatrix(order-2);
            tbs = new TextBox[10];
            tbx = new TextBox[10];

            InitializeComponent();
            b[0] = 1;
            for (i = 1; i <= ((order-1)/2); i++)
            {
                b[i] = CalcBinom(order-1, i);
                b[order-1 - i] = b[i];
            }
            b[order - 1] = 1;

            tbs[0] = textBoxY1; tbs[1] = textBoxY2; tbs[2] = textBoxY3; tbs[3] = textBoxY4; tbs[4] = textBoxY5;
            tbs[5] = textBoxY6; tbs[6] = textBoxY7; tbs[7] = textBoxY8; tbs[8] = textBoxY9; tbs[9] = textBoxY10;

            tbx[0] = textBoxX1; tbx[1] = textBoxX2; tbx[2] = textBoxX3; tbx[3] = textBoxX4; tbx[4] = textBoxX5;
            tbx[5] = textBoxX6; tbx[6] = textBoxX7; tbx[7] = textBoxX8; tbx[8] = textBoxX9; tbx[9] = textBoxX10;
 
            // init some sample y values
            y_k[0] = 160.0;  // set order = 4; 
            y_k[1] = 200.0;
            y_k[2] = 260.0;
            y_k[3] = 40.0;

            x_k[0] = 120.0;
            x_k[1] = 35.0;
            x_k[2] = 220.0;
            x_k[3] = 220.0;

            /* y_k[0] = 18.0; // set order = 10; 
             y_k[1] = 57.0;
             y_k[2] = 156.0;
             y_k[3] = 90.0;
             y_k[4] = 177.0;
             y_k[5] = 82.0;
             y_k[6] = 236.0;
             y_k[7] = 290.0;
             y_k[8] = 294.0;
             y_k[9] = 188.0;

             x_k[0] = 198.0;
             x_k[1] = 34.0;
             x_k[2] = 18.0;
             x_k[3] = 221.0;
             x_k[4] = 186.0;
             x_k[5] = 14.0;
             x_k[6] = 12.0;
             x_k[7] = 45.0;
             x_k[8] = 218.0;
             x_k[9] = 248.0; */
           

            if (order <= 10)
            {
                for (i = 0; i < order; i++)
                {
                    tbs[i].Text = Convert.ToString(y_k[i]);
                    tbx[i].Text = Convert.ToString(x_k[i]);

                }
                for (i = order; i < 10; i++)
                {
                    tbs[i].Visible = false;
                    tbx[i].Visible = false;
                }
            }
        }
          

        public void DrawGraph(int maxPoints, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double maxVal  = -1.0;
            double minVal  = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < order; j++)
            {
                if (maxTime < x_k[j])
                    maxTime = x_k[j];
                if (maxVal < y_k[j])
                    maxVal = y_k[j];
                if (minVal > y_k[j])
                    minVal = y_k[j];
            }
            for (j = 0; j < maxPoints - 1; j++)
            {
                if (maxTime < xp[j])
                    maxTime = xp[j];
                if (maxVal < yp[j])
                    maxVal = yp[j];
                if (minVal > yp[j])
                    minVal = yp[j];
            }
            maxTime = maxTime * 1.05;
            if (minVal < 0)
            {
                if (maxVal > 0)
                {
                    if (maxVal > Math.Abs(minVal))
                        scalefactor = pGraph.Height / maxVal / 2.2;
                    else
                        scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                }
                else
                    scalefactor = 5.0;
                drawCeroline = true;
            }
            else
            {
                scalefactor = pGraph.Height / (maxVal) / 1.1;
                drawCeroline = false;
            }
            p1 = new Point();
            p2 = new Point();
            Graphics g = pGraph.CreateGraphics();
            if (doClear)
                g.Clear(Color.White);
            Pen bluePen = new Pen(Color.Blue, 2);
            Pen redPen = new Pen(Color.Red, 2);
            Pen grayPen = new Pen(Color.LightGray, 2);
            grayPen.Width = 1;
            bluePen.Width = 1;

            p1.X = Convert.ToInt32(x_k[0] * pGraph.Width / (double)maxTime);
            if (drawCeroline)
                p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[0] * scalefactor));
            else
                p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[0] * scalefactor));

            for (j = 1; j < order; j++) // mark reference points
            {
                p2.X = Convert.ToInt32(x_k[j] * pGraph.Width / (double)maxTime);
                if (drawCeroline)
                    p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                else
                    p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                g.DrawLine(grayPen, p1, p2);
                p1.X = p2.X;
                p1.Y = p2.Y;
            }

            if (maxTime > 0) // draw approximated graph
            {
                for (j = 0; j <= maxPoints - 1; j++)
                {
                    p1.X = Convert.ToInt32(xp[j] * pGraph.Width / (double)maxTime);
                    p2.X = Convert.ToInt32(xp[j + 1] * pGraph.Width / (double)maxTime);
                    if (drawCeroline)
                    {
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                        p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                    }
                    else
                    {
                        p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                        p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                    }
                    g.DrawLine(bluePen, p1, p2);
                }              

                for (j = 0; j < order; j++) // mark reference points
                {
                    p1.X = Convert.ToInt32(x_k[j] * pGraph.Width / (double)maxTime);
                    if (drawCeroline)
                        p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    else
                        p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                    g.DrawEllipse(redPen, (int)(p1.X - 1.0), (int)(p1.Y - 1.0), 2, 2);
                }
            }
            g.Dispose();  
        }


        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(500, false);
        }

        private void button1_Click(object sender, EventArgs e)
        {
            int i;
            if (order < 10)
            {
                for (i = 0; i < order; i++)
                {
                    if (tbs[i].Text != "")
                    {
                        y_k[i] = Convert.ToDouble(tbs[i].Text);
                        x_k[i] = Convert.ToDouble(tbx[i].Text);
                    }
                    else
                    {
                        y_k[i] = 0.0;
                        x_k[i] = 0.0;
                    }
                }
            }
            Approximate(500);
            DrawGraph(500, true);
        }

    }
}
